@@ -1,4 +1,5 @@
 package Archive::Extract;
+use if $] > 5.017, 'deprecate';
 
 use strict;
 
@@ -16,6 +17,9 @@ use Locale::Maketext::Simple    Style => 'gettext';
 ### solaris has silly /bin/tar output ###
 use constant ON_SOLARIS     => $^O eq 'solaris' ? 1 : 0;
 use constant ON_NETBSD      => $^O eq 'netbsd' ? 1 : 0;
+use constant ON_OPENBSD     => $^O =~ m!^(openbsd|bitrig)$! ? 1 : 0;
+use constant ON_FREEBSD     => $^O =~ m!^(free|midnight|dragonfly)(bsd)?$! ? 1 : 0;
+use constant ON_LINUX       => $^O eq 'linux' ? 1 : 0;
 use constant FILE_EXISTS    => sub { -e $_[0] ? 1 : 0 };
 
 ### VMS may require quoting upper case command options
@@ -44,13 +48,13 @@ use vars qw[$VERSION $PREFER_BIN $PROGRAMS $WARN $DEBUG
             $_ALLOW_BIN $_ALLOW_PURE_PERL $_ALLOW_TAR_ITER
          ];
 
-$VERSION            = '0.50';
+$VERSION            = '0.74';
 $PREFER_BIN         = 0;
 $WARN               = 1;
 $DEBUG              = 0;
 $_ALLOW_PURE_PERL   = 1;    # allow pure perl extractors
 $_ALLOW_BIN         = 1;    # allow binary extractors
-$_ALLOW_TAR_ITER        = 1;    # try to use Archive::Tar->iter if available
+$_ALLOW_TAR_ITER    = 1;    # try to use Archive::Tar->iter if available
 
 # same as all constants
 my @Types           = ( TGZ, TAR, GZ, ZIP, BZ2, TBZ, Z, LZMA, XZ, TXZ );
@@ -125,12 +129,26 @@ See the C<HOW IT WORKS> section further down for details.
 
 ### see what /bin/programs are available ###
 $PROGRAMS = {};
-for my $pgm (qw[tar unzip gzip bunzip2 uncompress unlzma unxz]) {
-    if ( $pgm eq 'unzip' and ON_NETBSD ) {
+CMD: for my $pgm (qw[tar unzip gzip bunzip2 uncompress unlzma unxz]) {
+    if ( $pgm eq 'unzip' and ON_FREEBSD and my $unzip = can_run('info-unzip') ) {
+      $PROGRAMS->{$pgm} = $unzip;
+      next CMD;
+    }
+    if ( $pgm eq 'unzip' and ( ON_NETBSD or ON_FREEBSD ) ) {
       local $IPC::Cmd::INSTANCES = 1;
-      my @possibles = can_run($pgm);
-      ($PROGRAMS->{$pgm}) = grep { m!/usr/pkg/! } can_run($pgm);
-      next;
+      ($PROGRAMS->{$pgm}) = grep { ON_NETBSD ? m!/usr/pkg/! : m!/usr/local! } can_run($pgm);
+      next CMD;
+    }
+    if ( $pgm eq 'unzip' and ON_LINUX ) {
+      # Check if 'unzip' is busybox masquerading
+      local $IPC::Cmd::INSTANCES = 1;
+      my $opt = ON_VMS ? '"-Z"' : '-Z';
+      ($PROGRAMS->{$pgm}) = grep { scalar run(command=> [ $_, $opt, '-1' ]) } can_run($pgm);
+      next CMD;
+    }
+    if ( $pgm eq 'tar' and ( ON_OPENBSD || ON_SOLARIS || ON_NETBSD ) ) {
+      # try gtar first
+      next CMD if $PROGRAMS->{$pgm} = can_run('gtar');
     }
     $PROGRAMS->{$pgm} = can_run($pgm);
 }
@@ -257,16 +275,16 @@ Returns a C<Archive::Extract> object on success, or false on failure.
         ### figure out the type, if it wasn't already specified ###
         unless ( $parsed->{type} ) {
             $parsed->{type} =
-                $ar =~ /.+?\.(?:tar\.gz|tgz)$/i     ? TGZ   :
-                $ar =~ /.+?\.gz$/i                  ? GZ    :
-                $ar =~ /.+?\.tar$/i                 ? TAR   :
-                $ar =~ /.+?\.(zip|jar|par)$/i       ? ZIP   :
-                $ar =~ /.+?\.(?:tbz2?|tar\.bz2?)$/i ? TBZ   :
-                $ar =~ /.+?\.bz2$/i                 ? BZ2   :
-                $ar =~ /.+?\.Z$/                    ? Z     :
-                $ar =~ /.+?\.lzma$/                 ? LZMA  :
-                $ar =~ /.+?\.(?:txz|tar\.xz)$/i     ? TXZ   :
-                $ar =~ /.+?\.xz$/                   ? XZ    :
+                $ar =~ /.+?\.(?:tar\.gz|tgz)$/i         ? TGZ   :
+                $ar =~ /.+?\.gz$/i                      ? GZ    :
+                $ar =~ /.+?\.tar$/i                     ? TAR   :
+                $ar =~ /.+?\.(zip|jar|ear|war|par)$/i   ? ZIP   :
+                $ar =~ /.+?\.(?:tbz2?|tar\.bz2?)$/i     ? TBZ   :
+                $ar =~ /.+?\.bz2$/i                     ? BZ2   :
+                $ar =~ /.+?\.Z$/                        ? Z     :
+                $ar =~ /.+?\.lzma$/                     ? LZMA  :
+                $ar =~ /.+?\.(?:txz|tar\.xz)$/i         ? TXZ   :
+                $ar =~ /.+?\.xz$/                       ? XZ    :
                 '';
 
         }
@@ -646,7 +664,7 @@ sub have_old_bunzip2 {
         ### check for /bin/tar ###
         ### check for /bin/gzip if we need it ###
         ### if any of the binaries are not available, return NA
-        {   my $diag =  not $self->bin_tar ?
+        {   my $diag =  !$self->bin_tar ?
                             loc("No '%1' program found", '/bin/tar') :
                         $self->is_tgz && !$self->bin_gzip ?
                             loc("No '%1' program found", '/bin/gzip') :
@@ -669,17 +687,21 @@ sub have_old_bunzip2 {
         ### see what command we should run, based on whether
         ### it's a .tgz or .tar
 
+        ### GNU tar can't handled VMS filespecs, but VMSTAR can handle Unix filespecs.
+        my $archive = $self->archive;
+        $archive = VMS::Filespec::unixify($archive) if ON_VMS;
+
         ### XXX solaris tar and bsdtar are having different outputs
         ### depending whether you run with -x or -t
         ### compensate for this insanity by running -t first, then -x
         {    my $cmd =
-                $self->is_tgz ? [$self->bin_gzip, '-cdf', $self->archive, '|',
+                $self->is_tgz ? [$self->bin_gzip, '-c', '-d', '-f', $archive, '|',
                                  $self->bin_tar, '-tf', '-'] :
-                $self->is_tbz ? [$self->bin_bunzip2, '-cd', $self->archive, '|',
+                $self->is_tbz ? [$self->bin_bunzip2, '-cd', $archive, '|',
                                  $self->bin_tar, '-tf', '-'] :
-                $self->is_txz ? [$self->bin_unxz, '-cd', $self->archive, '|',
+                $self->is_txz ? [$self->bin_unxz, '-cd', $archive, '|',
                                  $self->bin_tar, '-tf', '-'] :
-                [$self->bin_tar, @ExtraTarFlags, '-tf', $self->archive];
+                [$self->bin_tar, @ExtraTarFlags, '-tf', $archive];
 
             ### run the command
             ### newer versions of 'tar' (1.21 and up) now print record size
@@ -696,12 +718,12 @@ sub have_old_bunzip2 {
             unless( $out[0] ) {
                 return $self->_error(loc(
                                 "Error listing contents of archive '%1': %2",
-                                $self->archive, $buffer ));
+                                $archive, $buffer ));
             }
 
             ### no buffers available?
             if( !IPC::Cmd->can_capture_buffer and !$buffer ) {
-                $self->_error( $self->_no_buffer_files( $self->archive ) );
+                $self->_error( $self->_no_buffer_files( $archive ) );
 
             } else {
                 ### if we're on solaris we /might/ be using /bin/tar, which has
@@ -728,13 +750,13 @@ sub have_old_bunzip2 {
 
         ### now actually extract it ###
         {   my $cmd =
-                $self->is_tgz ? [$self->bin_gzip, '-cdf', $self->archive, '|',
+                $self->is_tgz ? [$self->bin_gzip, '-c', '-d', '-f', $archive, '|',
                                  $self->bin_tar, '-xf', '-'] :
-                $self->is_tbz ? [$self->bin_bunzip2, '-cd', $self->archive, '|',
+                $self->is_tbz ? [$self->bin_bunzip2, '-cd', $archive, '|',
                                  $self->bin_tar, '-xf', '-'] :
-                $self->is_txz ? [$self->bin_unxz, '-cd', $self->archive, '|',
+                $self->is_txz ? [$self->bin_unxz, '-cd', $archive, '|',
                                  $self->bin_tar, '-xf', '-'] :
-                [$self->bin_tar, @ExtraTarFlags, '-xf', $self->archive];
+                [$self->bin_tar, @ExtraTarFlags, '-xf', $archive];
 
             my $buffer = '';
             unless( scalar run( command => $cmd,
@@ -742,7 +764,7 @@ sub have_old_bunzip2 {
                                 verbose => $DEBUG )
             ) {
                 return $self->_error(loc("Error extracting archive '%1': %2",
-                                $self->archive, $buffer ));
+                                $archive, $buffer ));
             }
 
             ### we might not have them, due to lack of buffers
@@ -946,7 +968,7 @@ sub _gunzip_bin {
         return $self->_error(loc("Could not open '%1' for writing: %2",
                             $self->_gunzip_to, $! ));
 
-    my $cmd = [ $self->bin_gzip, '-cdf', $self->archive ];
+    my $cmd = [ $self->bin_gzip, '-c', '-d', '-f', $self->archive ];
 
     my $buffer;
     unless( scalar run( command => $cmd,
@@ -1086,6 +1108,10 @@ sub _unzip_bin {
             $self->_error( $self->_no_buffer_files( $self->archive ) );
 
         } else {
+            ### Annoyingly, pesky MSWin32 can either have 'native' tools
+            ### which have \r\n line endings or Cygwin-based tools which
+            ### have \n line endings. Jan Dubois suggested using this fix
+            local $/ = ON_WIN32 ? qr/\r?\n/ : "\n";
             $self->files( [split $/, $buffer] );
         }
     }
@@ -1338,7 +1364,7 @@ sub _unxz_bin {
         return $self->_error(loc("Could not open '%1' for writing: %2",
                             $self->_gunzip_to, $! ));
 
-    my $cmd = [ $self->bin_unxz, '-cdf', $self->archive ];
+    my $cmd = [ $self->bin_unxz, '-c', '-d', '-f', $self->archive ];
 
     my $buffer;
     unless( scalar run( command => $cmd,
@@ -1646,7 +1672,7 @@ thread safe. See C<rt.cpan.org> bug C<#45671> for details.
 
 =head1 BUG REPORTS
 
-Please report bugs or other issues to E<lt>bug-archive-extract@rt.cpan.org<gt>.
+Please report bugs or other issues to E<lt>bug-archive-extract@rt.cpan.orgE<gt>.
 
 =head1 AUTHOR
 
